import org.vertx.java.core.Handler;
import org.vertx.java.core.SimpleHandler;
import org.vertx.java.core.http.HttpClient;
import org.vertx.java.core.http.HttpClientRequest;
import org.vertx.java.core.http.HttpClientResponse;
import org.vertx.java.core.http.HttpServerRequest;
import org.vertx.java.core.streams.Pump;
import org.vertx.java.deploy.Verticle;

/**
 * Proxies upload/download file requests.
 * Code is not consolidated: methods handleDownloadRequest and handleUploadRequest
 * should be consolidated.
 * @author mcalderoni
 */
public class Proxy extends Verticle {

   @Override
   public void start() {
      System.out.println("Started proxy on port " + Config.PROXY_PORT + ". Real Server's PORT is: " + Config.SERVER_PORT);
      //proxy client
      final HttpClient pClient = vertx.createHttpClient().setHost("localhost").setPort(Config.SERVER_PORT);
      //proxy server, intercepting requests from the real client
      vertx.createHttpServer().requestHandler(new Handler<HttpServerRequest>() {
         @Override
         public void handle(final HttpServerRequest psReq) {
            System.out.println("Proxy received " + psReq.method + " request (from client), " + psReq.uri);
            switch (psReq.method.toLowerCase()) {
               case "get":
                  handleDownloadRequest(psReq);
                  break;
               case "put":
                  handleUploadRequest(psReq);
                  break;
               default:
                  throw new UnsupportedOperationException("request method not supported");
            }
         }

         private void handleUploadRequest(final HttpServerRequest psReq) {
            psReq.pause();
            final HttpClientRequest pcReq = pClient.request(psReq.method, psReq.uri, new Handler<HttpClientResponse>() {
               @Override
               public void handle(HttpClientResponse pcResp) {
                  System.out.println("Proxy gets (from server) response: " + pcResp.statusCode);
                  //forward the response to the client
                  psReq.response.statusCode = pcResp.statusCode;
                  psReq.response.headers().putAll(pcResp.headers());
                  psReq.response.end();
               }
            });

            //forward the client request to the real server
            pcReq.headers().putAll(psReq.headers());
            pcReq.setChunked(true);
            psReq.endHandler(new SimpleHandler() {
               @Override
               public void handle() {
                  pcReq.end();
               }
            });
            Pump pump = Pump.createPump(psReq, pcReq);
            pump.start();
            psReq.resume();
         }

         private void handleDownloadRequest(final HttpServerRequest psReq) {
            final HttpClientRequest pcReq = pClient.request(psReq.method, psReq.uri, new Handler<HttpClientResponse>() {
               @Override
               public void handle(HttpClientResponse pcResp) {
                  pcResp.pause();
                  System.out.println("Proxy receives " + pcResp.statusCode + " response from server");
                  psReq.response.statusCode = pcResp.statusCode;
                  psReq.response.headers().putAll(pcResp.headers());
                  psReq.response.setChunked(true);
                  pcResp.endHandler(new SimpleHandler() {
                     @Override
                     public void handle() {
                        psReq.response.end();
                     }
                  });
                  Pump pump = Pump.createPump(pcResp, psReq.response);
                  pump.start();
                  pcResp.resume();
               }
            });

            pcReq.headers().putAll(psReq.headers());
            pcReq.end();
         }
      }).listen(Config.PROXY_PORT);
   }
}
