import java.util.UUID;
import org.vertx.java.core.AsyncResult;
import org.vertx.java.core.AsyncResultHandler;
import org.vertx.java.core.Handler;
import org.vertx.java.core.SimpleHandler;
import org.vertx.java.core.file.AsyncFile;
import org.vertx.java.core.http.HttpServerRequest;
import org.vertx.java.core.streams.Pump;
import org.vertx.java.deploy.Verticle;

/**
 * This server accepts uploads and downloads through put and get requests.
 * Run with: vertx run Server.java.
 */
public class Server extends Verticle {

   @Override
   public void start() {
      System.out.println("Started server on port " + Config.SERVER_PORT);

      vertx.createHttpServer().requestHandler(new Handler<HttpServerRequest>() {
         @Override
         public void handle(final HttpServerRequest req) {
            System.out.println("Received "+ req.method + " request, " + req.uri);
            switch (req.method.toLowerCase()) {
               case "get":
                  handleDownloadRequest(req);
                  break;
               case "put":
                  handleUploadRequest(req);
                  break;
               default:
                  throw new UnsupportedOperationException("request method not supported");
            }
         }
      }).listen(Config.SERVER_PORT);
   }

   private void handleUploadRequest(final HttpServerRequest req) {
      //We first pause the request so we don't receive any data between now and when the file is opened
      req.pause();
      final String filepath = Config.TARGET_FILE_REPO + UUID.randomUUID().toString() + ".upload";

      vertx.fileSystem().open(filepath, new AsyncResultHandler<AsyncFile>() {
         @Override
         public void handle(AsyncResult<AsyncFile> ar) {
            final AsyncFile asyncFile = ar.result;
            final Pump pump = Pump.createPump(req, asyncFile.getWriteStream());

            final long start = System.currentTimeMillis();
            req.endHandler(new SimpleHandler() {
               @Override
               public void handle() {
                  asyncFile.close(new AsyncResultHandler<Void>() {
                     @Override
                     public void handle(AsyncResult<Void> ar) {
                        if (ar.exception == null) {
                           req.response.statusCode = 200;
                           req.response.headers().putAll(req.headers());
                           req.response.end();
                           long end = System.currentTimeMillis();
                           int bytesPumped = pump.getBytesPumped();
                           System.out.println("\n\t\tUploaded " + pump.getBytesPumped() + " bytes\n\t\tto "
                                   + filepath + "\n\t\tin " + (end - start) + " ms");
                           if (Config.FILE_SIZE != bytesPumped) {
                              System.err.println("ERROR - LOST DATA --------------------------------");
                           }
                        } else {
                           ar.exception.printStackTrace(System.err);
                        }
                     }
                  });
               }
            });
            pump.start();
            req.resume();
         }
      });
   }

   private void handleDownloadRequest(final HttpServerRequest req) {
      //We first pause the request so we don't receive any data between now and when the file is opened
      req.pause();
      req.response.setChunked(true);

      vertx.fileSystem().open(Config.FILE, new AsyncResultHandler<AsyncFile>() {
         @Override
         public void handle(AsyncResult<AsyncFile> ar) {
            final AsyncFile asyncFile = ar.result;
            final Pump pump = Pump.createPump(asyncFile.getReadStream(), req.response);
            final long start = System.currentTimeMillis();
            asyncFile.getReadStream().endHandler(new SimpleHandler() {
               @Override
               public void handle() {
                  asyncFile.close(new AsyncResultHandler<Void>() {
                     @Override
                     public void handle(AsyncResult<Void> ar) {
                        if (ar.exception == null) {
                           long end = System.currentTimeMillis();
                           int bytesPumped = pump.getBytesPumped();
                           System.out.println("\n\tDownloaded " + pump.getBytesPumped()
                                   + " bytes\n\tto " + Config.FILE + "\n\tin " + (end - start) + " ms");
                           if (Config.FILE_SIZE != bytesPumped) {
                              req.response.statusMessage = "500";
                              System.err.println("ERROR - LOST DATA --------------------------------");
                           } else {
                              req.response.statusMessage = "200";
                           }
                        } else {
                           ar.exception.printStackTrace(System.err);
                           req.response.statusMessage = "500";
                        }
                        req.response.end();
                     }
                  });
               }
            });
            pump.start();
            req.resume();
         }
      });
   }
}
